#pragma once
#ifndef __GLWindowEx_H
#define __GLWindowEx_H

// Include style would ideally change to <GX/base.h>
#include "GXBase.h"
#include "glex.h"

//-----------------------------------------------------------------------------

/**
 * This class adds support for OpenGL extension functions to the GLWindow
 * class. Derive your own custom window class from GLWindowEx in order to
 * get support for OpenGL extensions.
 */
_GXBASE_BEGIN
class GLWindowEx :public GLWindow, public glex {
public:
	///> default constructor
	GLWindowEx() {
	}

	/**
	 * This event is handled so that the GL extension loader will load the
	 * OpenGL extensions automatically when the OpenGL context is created.
	 * NOTE: If you override OnCreate() in your derived class, it must call
	 * GLWindowEx::OnCreate(), or duplicate the code below.
	 */
	virtual void OnCreate() {
		// load extensions when the OpenGL drawing context is created
		glex::Load();
	}

private:
	///> copy is not supported
	GLWindowEx(const GLWindowEx &) {}

	///> assignment is not supported
	GLWindowEx & operator = (const GLWindowEx &) { return *this; }
};
_GXBASE_END

//-----------------------------------------------------------------------------

#endif//__GLWindowEx_H
